/*----------------------------------------------------------------------
copyright (c) 2008 Georget Fabien

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--------------------------------------------------------------------------
File : Highlighter.cpp
Program : gt2n
Date : 10/07/08
--------------------------------------------------------------------------
Nature : C++ file for the highlighter class
--------------------------------------------------------------------------*/
 #include <QtGui>

 #include "Highlighter.h"

 Highlighter::Highlighter(QTextDocument *parent): QSyntaxHighlighter(parent)
 {
     HighlightingRule rule;

     functionFormat.setFontItalic(true);
     //functionFormat.setForeground(Qt::blue);
     rule.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
     rule.format = functionFormat;
     highlightingRules.append(rule);


     //keywordFormat.setForeground(Qt::darkBlue);
     keywordFormat.setFontWeight(QFont::Bold);
     QStringList keywordPatterns;
     keywordPatterns << "\\babs\\b" << "\\basm\\b" << "\\bbool\\b" 
		      << "\\bbreak\\b" << "\\bbyte\\b" << "\\bcase\\b"
		      << "\\bchar\\b" << "\\bconst\\b" << "\\bcontinue\\b"
		      << "\\bdefault\\b" <<  "\\bdo\\b" << "\\belse\\b"
		      << "\\false\\b" << "\\bfor\\b" << "\\bgoto\\b"
		      << "\\bif\\b" << "\\binline\\b" << "\\bint\\b"
		      << "\\blong\\b" << "\\bmutex\\b" << "\\brepeat\\b"
		      << "\\breturn\\b" << "\\bsafecall\\b" << "\\bshort\\b"
		      << "\\bsign\\b" << "\\bstart\\b" << "\\bstring\\b"
		      << "\\bstruct\\b" << "\\bsub\\b" << "\\bswitch\\b"
		      << "\\btask\\b" << "\\btrue\\b" << "\\btypedef\\b"
		      << "\\bunsigned\\b" << "\\buntil\\b" << "\\bvoid\\b"
		      << "\\bwhile\\b" << "\\b__RETURN__\\b" << "\\b__RETVAL__\\b"
		      << "\\b__STRRETVAL__\\b" << "\\b__TMPBYTE__\\b" << "\\b__TMPWORD__\\b"
		      << "\\b__TMPLONG__\\b";
     foreach (QString pattern, keywordPatterns) {
         rule.pattern = QRegExp(pattern);
         rule.format = keywordFormat;
         highlightingRules.append(rule);
     }

  //Constant
     constantFormat.setForeground(Qt::darkGreen);
     QStringList constantPatterns;
     constantPatterns << "\\bTRUE\\b" << "\\bFALSE\\b" << "\\bNA\\b"
		      // SENSOR
		      << "\\bS[1-4]\\b" << "\\bIN_[1-4]\\b" << "\\bSENSOR_[1-4]\\b"
		      << "\\bSENSOR_TYPE_NONE\\b" << "\\bSENSOR_TYPE_TOUCH\\b" 
		      << "\\bSENSOR_TYPE_TEMPERATURE\\b" << "\\bSENSOR_TYPE_LIGHT\\b" 
		      << "\\bSENSOR_TYPE_ROTATION\\b" << "\\bSENSOR_TYPE_LIGHT_ACTIVE\\b" 
		      << "\\bSENSOR_TYPELIGHT_INACTIVE\\b" << "\\bSENSOR_TYPE_SOUND_DB\\b" 
		      << "\\bSENSOR_TYPE_SOUND_DBA\\b" << "\\bSENSOR_TYPE_CUSTOM\\b" 
		      << "\\bSENSOR_TYPE_LOWSPEED\\b" << "\\bSENSOR_TYPE_LOWSPEED_9V\\b" 
		      << "\\bSENSOR_TYPE_HIGHSPEED\\b"
		      << "\\bSENSOR_MODE_RAW\\b" << "\\bSENSOR_MODE_BOOL\\b" << "\\bSENSOR_MODE_EDGE\\b"
		      << "\\bSENSOR_MODE_PULSE\\b" << "\\bSENSOR_MODE_PURCENT\\b" << "\\bSENSOR_MODE_FAHRENHEIT\\b"
		      << "\\bSENSOR_MODE_CELSIUS\\b" << "\\bSENSOR_MODE_ROTATION\\b"
		      << "\\bSENSOR_TOUCH\\b" << "\\bSENSOR_LIGHT\\b" << "\\bSENSOR_ROTATION\\b"
		      << "\\bSENSOR_CELSIUS\\b" << "\\bSENSOR_FAHRENHEIT\\b" << "\\bSENSOR_PULSE\\b"
		      << "\\bSENSOR_EDGE\\b" 
		      //output
		      << "\\bUF_UPDATE_MODE\\b" << "\\bUF_UPDATE_SPEED\\b" << "\\bUF_UPDATE_TACHO_LIMIT\\b"
		      << "\\bUF_UPDATE_RESET_COUNT\\b" << "\\bUF_UPDATE_PID_VALUES\\b" 
		      << "\\bUF_UPDATE_RESET_BLOCK_COUNT\\b" << "\\bUF_UPDATE_RESET_ROTATION_COUNT\\b" 
		      << "\\bOUT_MODE_COAST\\b" << "\\bOUT_MODE_MOTORON\\b" << "\\bOUT_MODE_BRAKE\\b"
		      << "\\bOUT_MODE_REGULATED\\b" << "\\bOUT_MODE_REGMETHOD\\b" 
		      << "\\bOUT_RUNSTATE_IDLE\\b" << "\\bOUT_RUNSTATE_RAMPUP\\b" 
		      << "\\bOUT_RUNSTATE_RUNNING\\b" << "\\bOUT_RUNSTATE_RAMPDOWN\\b"
		      << "\\bOUT_REGMODE_IDLE\\b" << "\\bOUT_REGMODE_SPEED\\b" << "\\bOUT_REGMODE_SYNC\\b"
		      << "\\bRESET_NONE\\b" << "\\bRESET_COUNT\\b" << "\\bRESET_BLOCK_COUNT\\b"
		      << "\\bRESET_ROTATION_COUNT\\b" << "\\bRESET_BLACKANDTACHO\\b" << "\\bRESET_ALL\\b"
		      << "\\bOUT_[A-C]\\b" << "\\bOUT_AB\\b" << "\\bOUT_AC\\b"
		      << "\\bOUT_BC\\b" << "\\bOUT_ABC\\b" 
		      //ecran
		      << "\\bLCD_LINE[1-8]\\b"
		      // Tone
		      << "\\bTONE_A[3-7]\\b" << "\\bTONE_AS[3-7]\\b" << "\\bTONE_B[3-7]\\b"
		      << "\\bTONE_C[4-7]\\b" << "\\bTONE_CS[4-7]\\b" 
		      << "\\bTONE_D[4-7]\\b" << "\\bTONE_DS[4-7]\\b" << "\\bTONE_E[4-7]\\b"
		      << "\\bTONE_F[4-7]\\b" << "\\bTONE_FS[4-7]\\b" 
		      << "\\bTONE_G[4-7]\\b" << "\\bTONE_GS[4-7]\\b" 
		      // button
		      << "\\bBTN[1-4]\\b" << "\\bBTNEXIT\\b" << "\\bBTNRIGHT\\b" << "\\bBTNLEFT\\b" << "\\bBTNCENTER\\b"
		      // error 
		      << "\\bNO_ERR\\b";

     foreach (QString pattern, constantPatterns) {
         rule.pattern = QRegExp(pattern);
         rule.format = constantFormat;
         highlightingRules.append(rule);
     }


     APIfunctionFormat.setForeground(Qt::darkBlue);
     APIfunctionFormat.setFontWeight(QFont::Bold);
     QStringList ApiFunctionPatterns;
     ApiFunctionPatterns << "\\bWait+(?=\\()" << "\\bCurrentTick+(?=\\()"  << "\\bFirstTick+(?=\\()"
			  << "\\bSleepTimeout+(?=\\()" << "\\bSleepTimer+(?=\\()" << "\\bResetSleepTimer+(?=\\()" 
			 << "\\bSetSleepTimeout+(?=\\()" << "\\bSetSleepTimer+(?=\\()"  //Timing functions
			 //Program control Function
			 << "\\bStop+(?=\\()" << "\\bAcquire+(?=\\()" << "\\bRelease+(?=\\()" 
			 << "\\bPrecedes+(?=\\()" << "\\bFollows+(?=\\()" << "\\bExitTo+(?=\\()" 
			 << "\\bStrToNum+(?=\\()" << "\\bStrLen+(?=\\()" << "\\bStrIndex+(?=\\()"
			 << "\\bNumToStr+(?=\\()" << "\\bStrCat+(?=\\()" << "\\bSubStr+(?=\\()" 
			 << "\\bStrReplace+(?=\\()" << "\\bFlatten+(?=\\()" //String functions
			 << "\\bByteArrayToStr+(?=\\()" << "\\bByteArrayToStrEx+(?=\\()" << "\\bStrToByteArray+(?=\\()"
			 << "\\bArrayLen+(?=\\()" << "\\bArrayInit+(?=\\()" << "\\bArraySubset+(?=\\()" 
			 << "\\bArrayBuild+(?=\\()" // Array functions
			 << "\\bRandom+(?=\\()" << "\\bSqrt+(?=\\()" << "\\bSin+(?=\\()" 
			 << "\\bCos+(?=\\()" << "\\bAsin+(?=\\()" << "\\bAcos+(?=\\()" //Numeric functions 
			// sensor functions
			 << "\\bSensor+(?=\\()" << "\\bSensorUS+(?=\\()"
			 << "\\bSetSensor+(?=\\()" << "\\bSetSensorType+(?=\\()" << "\\bSetSensorMode+(?=\\()" 
			 << "\\bSetSensorLight+(?=\\()" << "\\bSetSensorSound+(?=\\()" << "\\bSetSensorTouch+(?=\\()" 
			 << "\\bSetSensorLowspeed+(?=\\()" << "\\bSetInput+(?=\\()" << "\\bClearSensor+(?=\\()" 
			 << "\\bResetSensor+(?=\\()" << "\\bSetCustomSensorZeroOffset+(?=\\()" 
			 << "\\bSetCustomSensorPercentFullScale(+(?=\\()" 
			 << "\\bSetCustomSensorActiveStatus+(?=\\()" << "\\bSetSensorDigiPinsDirection(+(?=\\()"  
			 << "\\bSetSensorDigiPinsStatus+(?=\\()" << "\\bSetSensorDigiPinsOutputLevel+(?=\\()" 
			 << "\\bSensor+(?=\\()" << "\\bSensorUs+(?=\\()" << "\\bSensorType+(?=\\()" 
			 << "\\bSensorMode+(?=\\()" << "\\bSensorRaw+(?=\\()" << "\\bsSensorNormalized+(?=\\()" 
			 << "\\bSensorScaled+(?=\\()" << "\\bSensorInvalid+(?=\\()" << "\\bSensorBoolean+(?=\\()" 
			 << "\\bGetInput+(?=\\()" << "\\bCustomSensorZeroOffset+(?=\\()"
			 << "\\bCustomSensorPercentFullScale+(?=\\()" << "\\bCustomSensorActiveStatus+(?=\\()" 
			 << "\\bSensorDigiPinsDirection+(?=\\()" << "\\bSensorDigiPinsStatus(+(?=\\()"
			 << "\\bSensorDigiPinsOutputLevel+(?=\\()" 
			 // Output functions
			 << "\\bOff+(?=\\()" << "\\bCoast+(?=\\()" << "\\bOffEx+(?=\\()" 
			 << "\\bCoastEx+(?=\\()" << "\\bFloat+(?=\\()" << "\\bOnFwd+(?=\\()" 
			 << "\\bOnFwdEx+(?=\\()" << "\\bOnRev+(?=\\()" << "\\bOnRevEx+(?=\\()"
			 << "\\bOnFwdReg+(?=\\()" << "\\bOnFwdRegEx+(?=\\()" << "\\bOnRevReg+(?=\\()" 
			 << "\\bOnRevRegEx+(?=\\()" << "\\bOnFwdSync+(?=\\()" << "\\bOnFwdSyncEx+(?=\\()" 
			 << "\\bOnRevSync+(?=\\()" << "\\bOnRevSyncEx+(?=\\()" << "\\bRotateMotor+(?=\\()" 
			 << "\\bRotateMotorPID+(?=\\()" << "\\bRotateMotorEx+(?=\\()"
			 << "\\bRotateMotorExPID+(?=\\()" << "\\bResetTachoCount+(?=\\()"
			 << "\\bResetBlockTachoCount+(?=\\()" << "\\bResetRotationcount+(?=\\()" 
			 << "\\bResetAllTachoCounts+(?=\\()" 
			 << "\\bSetOutput+(?=\\()" << "\\bGetOutput+(?=\\()" << "\\bMotorMode+(?=\\()" 
			 << "\\bMotorPower+(?=\\()" << "\\bMotorActualSpeed+(?=\\()" << "\\bMotorTachoCount+(?=\\()"
			 << "\\bMotorTachoLimit+(?=\\()" << "\\bMotorRunState+(?=\\()" << "\\bMotorTurnRatio+(?=\\()" 
			 << "\\bMotorRegulation+(?=\\()" << "\\bMotorOverload+(?=\\()" << "\\bMotorRegPValue+(?=\\()" 
			 << "\\bMotorRegIValue+(?=\\()" << "\\bMotorRegDValue+(?=\\()" 
			 << "\\bMotorBlockTachoCount+(?=\\()" << "\\bMotorRotationCount+(?=\\()"  
			 << "\\bMotorPwnFreq+(?=\\()" << "\\bSetMotorPwnFreq+(?=\\()" 
			 // IOMA 
			 << "\\bIOMA+(?=\\()" << "\\bSetIOMA+(?=\\()" 
			 // Sound Module
			 << "\\bPlayTone+(?=\\()" << "\\bPlayToneEx+(?=\\()"  
			 << "\\bPlayFile+(?=\\()" << "\\bPlayFileEx+(?=\\()"
			 << "\\bSoundFlags+(?=\\()" << "\\bSetSoundFlags+(?=\\()" << "\\bSoundState+(?=\\()" 
			 << "\\bSetSoundState+(?=\\()" << "\\bSoundMode+(?=\\()" << "\\bSetSoundMode+(?=\\()" 
			 << "\\bSoundFrequency+(?=\\()" << "\\bSetSoundFrequency+(?=\\()" << "\\bSoundDuration+(?=\\()" 
			 << "\\bSetSoundDuration+(?=\\()" << "\\bSoundSampleRate+(?=\\()"  
			 << "\\bSetSoundSampleRate+(?=\\()" << "\\bSoundVolume+(?=\\()" << "\\bSetsoundVolume+(?=\\()" 
			 << "\\bStopSound+(?=\\()"
			 // IOCTRL Module
			 << "\\bPowerDown+(?=\\()" << "\\bRebootInFirmwareMode+(?=\\()"
			 //display Module 
			 << "\\bNumOut+(?=\\()" << "\\bTextOut+(?=\\()" << "\\bGraphicOut+(?=\\()" 
			 << "\\bGraphicOutEx+(?=\\()" << "\\bCircleOut+(?=\\()" << "\\bLineOut+(?=\\()" 
			 << "\\bPointOut+(?=\\()" << "\\bRectOut+(?=\\()" << "\\bResetScreen+(?=\\()" 
			 << "\\bClearScreen+(?=\\()"  
			 << "\\bDisplayFlags+(?=\\()" << "\\bSetDisplayFlags+(?=\\()" << "\\bDisplayEraseMask+(?=\\()"
			 << "\\bSetDisplayEraseMask+(?=\\()" << "\\bDisplayupdateMask+(?=\\()" 
			 << "\\bSetDisplayUpdateMask+(?=\\()" << "\\bDisplayDisplay+(?=\\()"
			 << "\\bSetDisplayDisplay+(?=\\()" << "\\bDisplayTextLinesCenterFlags+(?=\\()" 
			 << "\\bSetDisplayTextLinesCenterFlags+(?=\\()" << "\\bGetDisplayNormal+(?=\\()"  
			 << "\\bSetDisplayNormal+(?=\\()" << "\\bGetDisplayPopup+(?=\\()" <<"\\bSetdisplayPoupup+(?=\\()"
			 //Loader module
			 << "\\bFreeMomory+(?=\\()" << "\\bCreateFile+(?=\\()" << "\\bOpenFileAppend+(?=\\()" 
			 << "\\bOpenFileRead+(?=\\()" << "\\bCloseFile+(?=\\()" << "\\bResolveHandle+(?=\\()" 
			 << "\\bRenameFile+(?=\\()" << "\\bDeleteFile+(?=\\()" << "\\bRead+(?=\\()" 
			 << "\\bReadLn+(?=\\()" << "\\bReadBytes+(?=\\()" << "\\bWrite+(?=\\()" 
			 << "\\bWriteLn+(?=\\()" << "\\bWriteString+(?=\\()" << "\\bWriteLnString+(?=\\()"
			 << "\\bWriteBytes+(?=\\()" << "\\bWriteBytesEx+(?=\\()" 
			 // Button Module
			 << "\\bButtonCount+(?=\\()" << "\\bButtonPressed+(?=\\()" << "\\bReadButtonEx+(?=\\()" 
			 << "\\bButtonPressCount+(?=\\()" << "\\bSetButtonPressCOunt+(?=\\()"  
			 << "\\bButtonLongPressCount+(?=\\()" << "\\bSetButtonLongPressCount+(?=\\()"
			 << "\\bButtonShortReleaseCount+(?=\\()" << "\\bSetButtonShortReleaseCount+(?=\\()" 
			 << "\\bButtonLongReleaseCount+(?=\\()" << "\\bSetButtonLongReleaseCount+(?=\\()"  
			 << "\\bButtonReleaseCount+(?=\\()" << "\\bSetButtonReleaseCount+(?=\\()" 
			 << "\\bButtonState+(?=\\()" << "\\bSetButtonState+(?=\\()"
			 // UI Module
			 << "\\bVolume+(?=\\()" << "\\bSetVolume+(?=\\()" << "\\bBatteryLevel+(?=\\()" 
			 << "\\bBluetoothState+(?=\\()" << "\\bSetBluetoothState+(?=\\()" 
			 << "\\bCommandFlags+(?=\\()"<< "\\bSetCommandFlags+(?=\\()" 
			 << "\\bUIState+(?=\\()"<< "\\b+SetUIState(?=\\()" << "\\bUIButton+(?=\\()"
			 << "\\bSetUIButton+(?=\\()"<< "\\bVMRunState+(?=\\()" << "\\bSetVMRunState+(?=\\()"
			 << "\\bBatteryState+(?=\\()"<< "\\bStateBatteryState+(?=\\()" <<"\\bRechargeableBattery+(?=\\()"
			 << "\\bForceOff+(?=\\()"<< "\\bUSBState+(?=\\()" << "\\bSetUSBState+(?=\\()"
			 << "\\bOnBrickProgramPointer+(?=\\()"<< "\\bSetOnBrickProgramPointer+(?=\\()"
			  // LowSpeed module
			 /*<< "\\b+(?=\\()"<< "\\b+(?=\\()" << "\\b+(?=\\()"
			 << "\\b+(?=\\()"<< "\\b+(?=\\()" << "\\b+(?=\\()"
			 << "\\b+(?=\\()"<< "\\b+(?=\\()" << "\\b+(?=\\()"*/;

     foreach (QString pattern, ApiFunctionPatterns) {
         rule.pattern = QRegExp(pattern);
         rule.format = APIfunctionFormat;
         highlightingRules.append(rule);
     }

     quotationFormat.setForeground(Qt::red);
     rule.pattern = QRegExp("\".*\"");
     rule.format = quotationFormat;
     highlightingRules.append(rule);

     singleLineCommentFormat.setForeground(Qt::gray);
     rule.pattern = QRegExp("//[^\n]*");
     rule.format = singleLineCommentFormat;
     highlightingRules.append(rule);

     preProcFormat.setForeground(Qt::darkMagenta);
     rule.pattern = QRegExp("#[^\n]*");
     rule.format = preProcFormat;
     highlightingRules.append(rule);     

     multiLineCommentFormat.setForeground(Qt::gray);

     commentStartExpression = QRegExp("/\\*");
     commentEndExpression = QRegExp("\\*/");
 }

 void Highlighter::highlightBlock(const QString &text)
 {
     foreach (HighlightingRule rule, highlightingRules) {
         QRegExp expression(rule.pattern);
         int index = text.indexOf(expression);
         while (index >= 0) {
             int length = expression.matchedLength();
             setFormat(index, length, rule.format);
             index = text.indexOf(expression, index + length);
         }
     }
     setCurrentBlockState(0);

     int startIndex = 0;
     if (previousBlockState() != 1)
         startIndex = text.indexOf(commentStartExpression);

     while (startIndex >= 0) {
         int endIndex = text.indexOf(commentEndExpression, startIndex);
         int commentLength;
         if (endIndex == -1) {
             setCurrentBlockState(1);
             commentLength = text.length() - startIndex;
         } else {
             commentLength = endIndex - startIndex
                             + commentEndExpression.matchedLength();
         }
         setFormat(startIndex, commentLength, multiLineCommentFormat);
         startIndex = text.indexOf(commentStartExpression,
                                                 startIndex + commentLength);
     }
 }
