/*----------------------------------------------------------------------
copyright (c) 2008 Georget Fabien

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--------------------------------------------------------------------------
File : InfoTab.h
Program : cwNXT
Date : 10/07/08
--------------------------------------------------------------------------
Nature : cpp file for the MainWindow class
--------------------------------------------------------------------------*/
 

 #include <QtGui>

 #include "MainWindow.h"
 
 #include "MdiChild.h"
 #include "InfoTab.h"
 #include "ExploTab.h"
 #include "CompDock.h"


 MainWindow::MainWindow()
 {
    // brick not connected
     isConnected = false;

    //create the window
     mdiArea = new QMdiArea;
 //    mdiArea->setViewMode(QMdiArea::TabbedView);    
     setCentralWidget(mdiArea);


     connect(mdiArea, SIGNAL(subWindowActivated(QMdiSubWindow *)),this, SLOT(updateMenus()));
     windowMapper = new QSignalMapper(this);
     connect(windowMapper, SIGNAL(mapped(QWidget *)), this, SLOT(setActiveSubWindow(QWidget *)));

     createDock();
     createActions();
     createMenus();
     createToolBars();
     createStatusBar();
     updateMenus();

     setWindowTitle(tr("cwNXT"));


 }

 void MainWindow::closeEvent(QCloseEvent *event)
 {
     mdiArea->closeAllSubWindows();
     if (activeMdiChild()) {
         event->ignore();
     } else {
         event->accept();
     }
 }



// connect the brick
void MainWindow::connectBrick()
{
   if (!isConnected) // check if the brick is already here 
    {
      zeBrick = new UsbNxt; 
     QApplication::setOverrideCursor(Qt::WaitCursor);
      if (zeBrick->init() != 0) //initialize brick
      {
	  // Not OK
	  afficherMessage("Error : no NXT brick found");
	  QMessageBox::critical(this, "Error", "No NXT brick found");
	  isConnected = false;
	  infoDock->effacerInfo();
	  infoDock->actEnableConnect(false);
	  exploDock->clean();
	  exploDock->actEnableConnect(false);
	  actEnableConnect(false);
      }
      else
      {
	  // OK
	  afficherMessage("NXT brick found, USB initialized");
	  isConnected = true;
	  infoDock->getInfo();
	  infoDock->actEnableConnect(true);
	  exploDock->listFile();
	  exploDock->actEnableConnect(true);
	  actEnableConnect(true);
      }
      QApplication::restoreOverrideCursor();
    }
    else
    {
      // If the brick is already here
      QMessageBox::critical(this, "Error", "Brick already connect !");
    }
}

// rename the brick
void MainWindow::renameBrick()
{
  if (brickIsHere())
  {
      bool ok = FALSE;
      QString newName = QInputDialog::getText(this, "New Name", "What is the new name of the brick ?", QLineEdit::Normal, QString(), &ok);
      if (ok && !newName.isEmpty())
      {
	const char*(nom) = newName.toAscii(); // permet de transformer une QString en un tableau de char
	zeBrick->setname(nom);
      }
      else
      {
	afficherMessage("Action cancelled !");
      }
  }
  getInfo();
}

// return information about the brick
nxt_info MainWindow::getInfo()
{
  if(brickIsHere())
  {
    nxt_info info;
    zeBrick->get_info(&info);
    return info;
  }
}


// start the process to list files
nxt_file MainWindow::getFirstFile()
{
    nxt_file fileDesc;
    fileDesc.status = 1; // pour arreter si brique non brancher
    if(brickIsHere()){ 
      zeBrick->getFirstFile(&fileDesc);
      return fileDesc;
    }
}

// return a file's description
// take part of the process to list files
nxt_file MainWindow::getNextFile(char handle)
{
    nxt_file fileDesc;
    fileDesc.status = 1; // pour arreter si brique non brancher
    if(brickIsHere()){ 
      zeBrick->getNextFile(&fileDesc,handle);
      return fileDesc;
    }
}

// close the process to list files
void MainWindow::closeSearchFile(char handle)
{
    if(brickIsHere()){ 
      zeBrick->closeSearchFile(handle);
    }
}

// delete a file on the brick
void MainWindow::deleteFile(QString filename)
{
  std::string file = filename.toStdString();
  const char* nom = file.c_str();
  int res = zeBrick->fdelete(nom);
  if (res==-1) {messError(tr("file")+filename+tr("can't be deleted"),tr("deleting file"));}
  exploDock->listFile();
}

// download a file
// the parameter can be a path 
void MainWindow::downloadFile(QString fpath)
{
  if(brickIsHere())
  {	
    int res = zeBrick->download(fpath.toAscii());
    switch(res)
    {
      case -1: // filename too long
	messError(tr("Filename must be less than 15 chars"),tr("downloading file"));
	break;
      case -2: // no such file on the brick
	messError(tr("No such file on the brick"),tr("downloading file"));
	break;
      case -3: // can't open the local file
	messError(tr("Create local file failed"),tr("downloading file"));
	break;
      case -4: //can't open remote file
	messError(tr("Open remote file failed"),tr("downloading file"));
	break;
      case -5: //write local file failed
	messError(tr("Write local file failed"),tr("downloading file"));
	break;
      case -6: //error while writing
	messError(tr("Error while writing"),tr("downloading file"));
	break;
      case -7: //bad communication
	messError(tr("Error in the communication"),tr("downloading file"));
	break; 
      default:
	afficherMessage(tr("Download succesful"));
	break;
    }
  }
}

//upload a file
// the parameter can be a path 
void MainWindow::uploadFile(QString fpath)
{
  if (brickIsHere()) 
  {
    const char*(nom) = fpath.toAscii(); // permet de transformer une QString en un tableau de char
    int res = zeBrick->upload(nom);
    switch(res)
    {
      case -1: // filename too long
	messError(tr("Filename must be less than 15 chars"),tr("uploading file"));
	break;
      case -2: // Can't load the file, size<0
	messError(tr("Can't load the file"),tr("uploading file"));
	break;
      case -3: // can't initiate uploate
	messError(tr("can't initiate upload"),tr("uploading file"));
	break;
      case -4: //error while writing
	messError(tr("Write file failed"),tr("uploading file"));
	break;
      case -7: //bad communication
	messError(tr("Error in the communication"),tr("uploading file"));
	break;   
      case 1: //File already exists
	int doIoverwrite = QMessageBox::question(this, "File already exists", "This file alreday exists on the brick. Do you want to overwrite it ?", QMessageBox::Yes|QMessageBox::No);
	if (doIoverwrite==QMessageBox::Yes)
	{
	  std::string fname;
	  std::string ffpath = fpath.toStdString();
	
	  size_t i = ffpath.rfind('/',ffpath.length());
	      if (i != std::string::npos) {
		 fname =  ffpath.substr(i+1,ffpath.length()-i);
	      }
	  QString str = fname.c_str();
	  deleteFile(str);
	  exploDock->listFile();
	  uploadFile(fpath);
	}
	else
	{
	  afficherMessage("Action cancelled !");
	}
    }
   exploDock->listFile();
  }

}

// return true if the brick is connected and on !
bool  MainWindow::brickIsHere()
{
   if(isConnected && zeBrick->brickIsHere())
   { return true ;}
   else 
   {
    int doIconnect = QMessageBox::question(this, "Error", "You need to connect a brick before this action ! Do you want to connect it ?", QMessageBox::Yes|QMessageBox::No);
    if (doIconnect==QMessageBox::Yes)
    {
      connectBrick();
      if (isConnected) {return zeBrick->brickIsHere();}
      else return false;
    }
    else
    {
      afficherMessage("action cancelled !");
      isConnected = false;
      infoDock->effacerInfo();
      infoDock->actEnableConnect(false);
      exploDock->clean();
      exploDock->actEnableConnect(false);
      actEnableConnect(false);
      return false;
    }
   }
}


/* -----------------------------------------------------------------------*\
|			       SLOT                                        |
\*------------------------------------------------------------------------*/

// create a new file (so a new MdiChild => new QTextEdit)
 void MainWindow::newFile()
 {
     MdiChild *child = createMdiChild();
     child->newFile();
     child->show();
 }

// open an existing file in a new MdiChild
// if it can't open this file, MdiChild is deleted
 void MainWindow::open()
 {
     QString fileName = QFileDialog::getOpenFileName(this);
     if (!fileName.isEmpty()) {
         QMdiSubWindow *existing = findMdiChild(fileName);
         if (existing) {
             mdiArea->setActiveSubWindow(existing);
             return;
         }

         MdiChild *child = createMdiChild();
         if (child->loadFile(fileName)) {
             afficherMessage(tr("File loaded"), 2000);
             child->show();
         } else {
             child->close();
         }
     }
 }

 void MainWindow::save()
 {
     if (activeMdiChild() && activeMdiChild()->save())
         afficherMessage(tr("File saved"), 2000);
 }

 void MainWindow::saveAs()
 {
     if (activeMdiChild() && activeMdiChild()->saveAs())
         afficherMessage(tr("File saved"), 2000);
 }

 void MainWindow::cut()
 {
     if (activeMdiChild())
         activeMdiChild()->cut();
 }

 void MainWindow::copy()
 {
     if (activeMdiChild())
         activeMdiChild()->copy();
 }

 void MainWindow::paste()
 {
     if (activeMdiChild())
         activeMdiChild()->paste();
 }

 void MainWindow::undo()
 {
     if (activeMdiChild())
        activeMdiChild()->undo();
 }
  
 void MainWindow::redo()
 {
     if (activeMdiChild())
        activeMdiChild()->redo();
 }

 void MainWindow::about()
 {
    QDialog fenetre(this);
    QTabWidget *tabu = new QTabWidget(&fenetre);
    tabu->setGeometry(30, 20, 400, 200);

    QLabel *cwnxt = new QLabel(" <strong>cwNXT</strong> : IDE for LEGO NXT Brick <br/><br/> Copyright (c) 2008 Georget Fabien <br /> Please report bugs at : manawy@nxttransfer.societeg.com <br /> <br/>License : GPLv3 <br /> available here : <a href=\"http://www.gnu.org/licenses/gpl-3.0.html\">http://www.gnu.org/licenses/gpl-3.0.html</a>");
    QLabel *thanks = new QLabel("<strong>John Hansen</strong> :<br /> for writting the nbc/nxc languages <br /> <br /> <strong>Pascal Raymond</strong> :<br /> for writting talk2nxt, cwNXT is based from this program");
    tabu->addTab(cwnxt,tr("cwNXT"));
    tabu->addTab(thanks,tr("thanks"));
    fenetre.show();
    fenetre.exec();
 }

// change the status of syntax highlighter for the active MdiChild
void MainWindow::highlight()
{
  if (activeMdiChild())
  {  
    activeMdiChild()->highlight();
  }
}

// Update menus when an other MdiChild is actived 
 void MainWindow::updateMenus()
 {
     bool hasMdiChild = (activeMdiChild() != 0);
     saveAct->setEnabled(hasMdiChild);
     saveAsAct->setEnabled(hasMdiChild); 
     undoAct->setEnabled(hasMdiChild);
     redoAct->setEnabled(hasMdiChild);
     pasteAct->setEnabled(hasMdiChild);
     closeAct->setEnabled(hasMdiChild);
     closeAllAct->setEnabled(hasMdiChild);
     tileAct->setEnabled(hasMdiChild);
     cascadeAct->setEnabled(hasMdiChild);
     nextAct->setEnabled(hasMdiChild);
     previousAct->setEnabled(hasMdiChild);
     separatorAct->setVisible(hasMdiChild);

     bool hasSelection = (activeMdiChild() && activeMdiChild()->textCursor().hasSelection());
     cutAct->setEnabled(hasSelection);
     copyAct->setEnabled(hasSelection);

     if (activeMdiChild())
     {	
	highlightAct->setChecked(activeMdiChild()->hasHighlight());
      }
 }

//update window menu when user click on the window menu button 
 void MainWindow::updateWindowMenu()
 {
     windowMenu->clear();
     windowMenu->addAction(closeAct);
     windowMenu->addAction(closeAllAct);
     windowMenu->addSeparator();
     windowMenu->addAction(tileAct);
     windowMenu->addAction(cascadeAct);
     windowMenu->addSeparator();
     windowMenu->addAction(nextAct);
     windowMenu->addAction(previousAct);
     windowMenu->addAction(separatorAct);

     QList<QMdiSubWindow *> windows = mdiArea->subWindowList();
     separatorAct->setVisible(!windows.isEmpty());

     for (int i = 0; i < windows.size(); ++i) {
         MdiChild *child = qobject_cast<MdiChild *>(windows.at(i)->widget());

         QString text;
         if (i < 9) {
             text = tr("&%1 %2").arg(i + 1).arg(child->userFriendlyCurrentFile());
         } else {
             text = tr("%1 %2").arg(i + 1).arg(child->userFriendlyCurrentFile());
         }
         QAction *action  = windowMenu->addAction(text);
         action->setCheckable(true);
         action->setChecked(child == activeMdiChild());
         connect(action, SIGNAL(triggered()), windowMapper, SLOT(map()));
         windowMapper->setMapping(action, windows.at(i));
     }
 }


// slot to connect the brick
void MainWindow::s_connectBrick() 
{
    connectBrick();
}

// slot to rename the brick
void MainWindow::s_renameBrick()
{
  renameBrick(); // jsut call the function
}

// SLOT to upload a file
void MainWindow::s_uploadFile() 
{
  if (brickIsHere()) 
  {
    QString fichier = QFileDialog::getOpenFileName(this, tr("UploadFile"), QDir::homePath());
    uploadFile(fichier);
    
  }
}

// disconnect the brick == handle variable
void MainWindow::s_disconnectBrick()
{
    if (isConnected)
    {	
      delete zeBrick;
      isConnected = false;
      infoDock->effacerInfo();
      infoDock->actEnableConnect(false);
      exploDock->clean();
      exploDock->actEnableConnect(false);
      actEnableConnect(false);
    }
}

// compile the file on the active Mdi Child
void MainWindow::s_compile()
{
    if (activeMdiChild()){
      compDock->compile(QFileInfo(activeMdiChild()->currentFile()));
    }
    else {
      QString fileName = QFileDialog::getOpenFileName(this, tr("Compile"),QDir::homePath());
     if (!fileName.isEmpty())
      compDock->compile(QFileInfo(fileName));
    }
}

// transfert a file : compile and upload it
void MainWindow::s_transfert()
{
    QFileInfo fileInfo;
    if (activeMdiChild()){
       fileInfo = QFileInfo(activeMdiChild()->currentFile());
    }
    else {
     QString fileName = QFileDialog::getOpenFileName(this, tr("Compile"),QDir::homePath());
     if (fileName.isEmpty()){ return;}
      fileInfo = QFileInfo(fileName); 
    }
    compDock->compile(fileInfo);
     if (brickIsHere())
      {
	if (compDock->readyToTransfer()){uploadFile(fileInfo.absolutePath()+"/"+fileInfo.completeBaseName()+".rxe");}
	else {messError(tr("Compilation aborted : no file to upload"),tr("uploading file"));}
    }
}

/* -----------------------------------------------------------------------*\
|			      Private                                       |
\*------------------------------------------------------------------------*/

// create a new MdiChild
 MdiChild *MainWindow::createMdiChild()
 {
     MdiChild *child = new MdiChild(this);
     mdiArea->addSubWindow(child);

     connect(child, SIGNAL(copyAvailable(bool)),cutAct, SLOT(setEnabled(bool)));
     connect(child, SIGNAL(copyAvailable(bool)),copyAct, SLOT(setEnabled(bool)));
     connect(child, SIGNAL(undoAvailable(bool)),undoAct, SLOT(setEnabled(bool)));
     connect(child, SIGNAL(redoAvailable(bool)),redoAct, SLOT(setEnabled(bool)));
     return child;
 }

 void MainWindow::setActiveSubWindow(QWidget *window)
 {
     if (!window)
         return;
     mdiArea->setActiveSubWindow(qobject_cast<QMdiSubWindow *>(window));
 }

// create the dock widgets :
// - information : QLabels
// - explorer : QListView
// - Build lod : QTextEdit
void MainWindow::createDock()
{
  QDockWidget *dock = new QDockWidget(tr("information"),this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  infoDock = new InfoTab(dock,this);
  dock->setWidget(infoDock);
  addDockWidget(Qt::RightDockWidgetArea, dock);

  dock = new QDockWidget(tr("Explorer"),this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  exploDock = new ExploTab(dock,this);
  dock->setWidget(exploDock);
  addDockWidget(Qt::RightDockWidgetArea, dock);

  dock = new QDockWidget(tr("Build log"),this);
  dock->setAllowedAreas(Qt::BottomDockWidgetArea);
  dock->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
  // can't close the build log => avoid errors
  compDock = new CompDock(this);
  dock->setWidget(compDock);
  addDockWidget(Qt::BottomDockWidgetArea, dock);

  
}

// display a message on the status bar
void MainWindow::afficherMessage(QString message,int time)
{
    barreEtat->showMessage(message,time);
}

// the message displayed for errors
void MainWindow::messError(QString message,QString action)
{
  QMessageBox::critical(this, tr("Error"), tr("Error while ")+action+tr(":\n")+message);
  afficherMessage(tr("Error while : ") + action);
}


// a method to enable/disable action 
// true when connecting brick
// false when disconecting brick
void MainWindow::actEnableConnect(bool boolean)
{
  actionDeconnect->setEnabled(boolean);
  actionRefresh->setEnabled(boolean);
  actionRename->setEnabled(boolean);
  actionUpload->setEnabled(boolean);
  actionDelete->setEnabled(boolean);
  
  actionConnect->setEnabled(!boolean);
}

 void MainWindow::createActions()
 {
     newAct = new QAction( tr("&New"), this);
     newAct->setShortcut(tr("Ctrl+N"));
     newAct->setStatusTip(tr("Create a new file"));
     connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

     openAct = new QAction(tr("&Open..."), this);
     openAct->setShortcut(tr("Ctrl+O"));
     openAct->setStatusTip(tr("Open an existing file"));
     connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

     saveAct = new QAction( tr("&Save"), this);
     saveAct->setShortcut(tr("Ctrl+S"));
     saveAct->setStatusTip(tr("Save the document to disk"));
     connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

     saveAsAct = new QAction(tr("Save &As..."), this);
     saveAsAct->setStatusTip(tr("Save the document under a new name"));
     connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

     exitAct = new QAction(tr("E&xit"), this);
     exitAct->setShortcut(tr("Ctrl+Q"));
     exitAct->setStatusTip(tr("Exit the application"));
     connect(exitAct, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

     undoAct = new QAction(tr("&Undo"), this);
     undoAct->setShortcut(tr("Ctrl+Z"));
     undoAct->setStatusTip(tr("Undo"));
     connect(undoAct, SIGNAL(triggered()), this, SLOT(undo()));

     redoAct = new QAction(tr("Re&do"), this);
     redoAct->setShortcut(tr("Ctrl+Shift+Z"));
     redoAct->setStatusTip(tr("Redo"));
     connect(redoAct, SIGNAL(triggered()), this, SLOT(redo()));

     cutAct = new QAction(tr("Cu&t"), this);
     cutAct->setShortcut(tr("Ctrl+X"));
     cutAct->setStatusTip(tr("Cut the current selection's contents to the clipboard"));
     connect(cutAct, SIGNAL(triggered()), this, SLOT(cut()));

     copyAct = new QAction(tr("&Copy"), this);
     copyAct->setShortcut(tr("Ctrl+C"));
     copyAct->setStatusTip(tr("Copy the current selection's contents to the clipboard"));
     connect(copyAct, SIGNAL(triggered()), this, SLOT(copy()));

     pasteAct = new QAction(tr("&Paste"), this);
     pasteAct->setShortcut(tr("Ctrl+V"));
     pasteAct->setStatusTip(tr("Paste the clipboard's contents into the current "
                               "selection"));
     connect(pasteAct, SIGNAL(triggered()), this, SLOT(paste()));

     closeAct = new QAction(tr("Cl&ose"), this);
     closeAct->setShortcut(tr("Ctrl+F4"));
     closeAct->setStatusTip(tr("Close the active window"));
     connect(closeAct, SIGNAL(triggered()),
             mdiArea, SLOT(closeActiveSubWindow()));

     closeAllAct = new QAction(tr("Close &All"), this);
     closeAllAct->setStatusTip(tr("Close all the windows"));
     connect(closeAllAct, SIGNAL(triggered()),
             mdiArea, SLOT(closeAllSubWindows()));

     tileAct = new QAction(tr("&Tile"), this);
     tileAct->setStatusTip(tr("Tile the windows"));
     connect(tileAct, SIGNAL(triggered()), mdiArea, SLOT(tileSubWindows()));

     cascadeAct = new QAction(tr("&Cascade"), this);
     cascadeAct->setStatusTip(tr("Cascade the windows"));
     connect(cascadeAct, SIGNAL(triggered()), mdiArea, SLOT(cascadeSubWindows()));

     nextAct = new QAction(tr("Ne&xt"), this);
     nextAct->setStatusTip(tr("Move the focus to the next window"));
     connect(nextAct, SIGNAL(triggered()), mdiArea, SLOT(activateNextSubWindow()));

     previousAct = new QAction(tr("Pre&vious"), this);
     previousAct->setStatusTip(tr("Move the focus to the previous window"));
     connect(previousAct, SIGNAL(triggered()), mdiArea, SLOT(activatePreviousSubWindow()));

     separatorAct = new QAction(this);
     separatorAct->setSeparator(true);

     aboutAct = new QAction(tr("&About"), this);
     aboutAct->setStatusTip(tr("Show the cwNXT's About box"));
     connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

     aboutQtAct = new QAction(tr("About &Qt"), this);
     aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
     connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

      //=> syntaxhighlighting or not 
     highlightAct = new QAction(tr("Highlighting"), this);
     highlightAct->setCheckable(true);
     highlightAct->setChecked(true);
     highlightAct->setStatusTip(tr("Enable syntax highlighter"));
     connect(highlightAct, SIGNAL(triggered()), this, SLOT(highlight()));

        //=> connect the brick
      actionConnect = new QAction(tr("&Connect","connect the brick"),this);
      actionConnect->setStatusTip(tr("Connect the brick"));
      connect(actionConnect, SIGNAL(triggered()),this, SLOT(s_connectBrick()));

    //=> disconnect the brick
      actionDeconnect = new QAction(tr("&Disconnect"),this);
      actionDeconnect->setStatusTip(tr("Disconnect the brick"));
      actionDeconnect->setEnabled(false);
      QObject::connect(actionDeconnect, SIGNAL(triggered()), this, SLOT(s_disconnectBrick()));

    //=>actionRefresh => refresh information
      actionRefresh = new QAction(tr("&Refresh"),this);
      actionRefresh->setStatusTip(tr("Refresh information"));
      actionRefresh->setEnabled(false);
      QObject::connect(actionRefresh, SIGNAL(triggered()), infoDock, SLOT(s_getInfo()));
      QObject::connect(actionRefresh, SIGNAL(triggered()), exploDock, SLOT(s_listFile()));

    //=> actionRename => rename the brick
     actionRename = new QAction(tr("Re&name"),this);
     actionRename->setStatusTip(tr("Rename the brick"));
     actionRename->setEnabled(false);
     QObject::connect(actionRename, SIGNAL(triggered()), this, SLOT(s_renameBrick()));

    //=> actionUpload => Upload a file on the brick
      actionUpload = new QAction(tr("&Upload a file"),this);
      actionUpload->setStatusTip(tr("Upload a file on the brick"));
      actionUpload->setEnabled(false);
      QObject::connect(actionUpload, SIGNAL(triggered()), this, SLOT(s_uploadFile()));

    //=> actionDelete => delete file on the brick
      actionDelete = new QAction(tr("&Delete file"),this);
      actionDelete->setStatusTip(tr("Delete file on the brick"));
      actionDelete->setEnabled(false);      
      QObject::connect(actionDelete, SIGNAL(triggered()), exploDock, SLOT(s_delFile()));
      
    //=> actionCompile => compile a file
      actionCompile = new QAction(tr("&Compile"),this);
      actionCompile->setStatusTip(tr("Compile current file"));
      QObject::connect(actionCompile, SIGNAL(triggered()), this, SLOT(s_compile()));

    //=> actionCompile => compile a file
      actionTransfert = new QAction(tr("&Transfer"),this);
      actionTransfert->setStatusTip(tr("Compile and upload on the Brick current file"));
      QObject::connect(actionTransfert, SIGNAL(triggered()), this, SLOT(s_transfert()));  

    //=> clear log
      actionCleanLog = new QAction(tr("&Clean log"),this);
      actionCleanLog->setStatusTip(tr("Clean build log"));
      QObject::connect(actionCleanLog, SIGNAL(triggered()), compDock, SLOT(s_cleanLog()));

    //=> save log
      actionSaveLog = new QAction(tr("&Save log"),this);
      actionSaveLog->setStatusTip(tr("Save build log"));
      QObject::connect(actionSaveLog, SIGNAL(triggered()), compDock, SLOT(s_saveLog()));
  
 }

 void MainWindow::createMenus()
 {
     fileMenu = menuBar()->addMenu(tr("&File"));
     fileMenu->addAction(newAct);
     fileMenu->addAction(openAct);
     fileMenu->addAction(saveAct);
     fileMenu->addAction(saveAsAct);
     fileMenu->addSeparator();
     fileMenu->addAction(exitAct);

     editMenu = menuBar()->addMenu(tr("&Edit")); 
     editMenu->addAction(undoAct);
     editMenu->addAction(redoAct);
     editMenu->addSeparator();
     editMenu->addAction(cutAct);
     editMenu->addAction(copyAct);
     editMenu->addAction(pasteAct);
     editMenu->addSeparator();
     editMenu->addAction(highlightAct);

    menuBar()->addSeparator();

    brickMenu = menuBar()->addMenu(tr("&Brick"));
      brickMenu->addAction(actionConnect);
      brickMenu->addAction(actionDeconnect);
      brickMenu->addSeparator();
      brickMenu->addAction(actionRefresh);
      brickMenu->addAction(actionRename);

    explorerMenu = menuBar()->addMenu(tr("E&xplorer"));
      explorerMenu->addAction(actionUpload);
      explorerMenu->addAction(actionDelete);
      
    compilerMenu = menuBar()->addMenu(tr("&Compiler"));
      compilerMenu->addAction(actionCompile);
      compilerMenu->addAction(actionTransfert);
      compilerMenu->addSeparator();
      compilerMenu->addAction(actionCleanLog);
      compilerMenu->addAction(actionSaveLog);

     menuBar()->addSeparator();

     windowMenu = menuBar()->addMenu(tr("&Window"));
     updateWindowMenu();
     connect(windowMenu, SIGNAL(aboutToShow()), this, SLOT(updateWindowMenu()));

     helpMenu = menuBar()->addMenu(tr("&Help"));
     helpMenu->addAction(aboutAct);
     helpMenu->addAction(aboutQtAct);
 }

 void MainWindow::createToolBars()
 {
     fileToolBar = addToolBar(tr("File"));
     fileToolBar->addAction(newAct);
     fileToolBar->addAction(openAct);
     fileToolBar->addAction(saveAct);

     editToolBar = addToolBar(tr("Edit"));
     editToolBar->addAction(cutAct);
     editToolBar->addAction(copyAct);
     editToolBar->addAction(pasteAct);
     
     brickToolBar = addToolBar(tr("Brick"));
     brickToolBar->addAction(actionConnect);
     brickToolBar->addAction(actionDeconnect);
     brickToolBar->addAction(actionRefresh);
 }

// create the status bar
void MainWindow::createStatusBar()
{
    barreEtat = statusBar();
    barreEtat->showMessage(tr("Welcome on cwNXT"),2000);
    labelEtat = new QLabel(tr("Ready"));
    barreEtat->addWidget(labelEtat);
}

// Return the active MdiChild
 MdiChild *MainWindow::activeMdiChild()
 {
     if (QMdiSubWindow *activeSubWindow = mdiArea->activeSubWindow())
         return qobject_cast<MdiChild *>(activeSubWindow->widget());
     return 0;
 }

// Find an MdiChild => to not open twice the same file
 QMdiSubWindow *MainWindow::findMdiChild(const QString &fileName)
 {
     QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();

     foreach (QMdiSubWindow *window, mdiArea->subWindowList()) {
         MdiChild *mdiChild = qobject_cast<MdiChild *>(window->widget());
         if (mdiChild->currentFile() == canonicalFilePath)
             return window;
     }
     return 0;
 }

